"use strict";
const utils_1 = require("../utils");
const hasOnlyWhitespace = (value) => /^[\r\n\s\t\f\v]+$/.test(value);
function isValidValue(value, config) {
    return (typeof value !== 'string' ||
        hasOnlyWhitespace(value) ||
        config.ignorePattern.test(value.trim()) ||
        config.ignoreText.includes(value.trim()));
}
function checkSvelteMustacheTagText(context, node, config) {
    if (!node.expression || !node.parent) {
        return;
    }
    if (node.parent.type === 'SvelteElement') {
        if (node.expression.type === 'Literal') {
            const literalNode = node.expression;
            if (isValidValue(literalNode.value, config)) {
                return;
            }
            context.report({
                node: literalNode,
                message: `raw text '${literalNode.value}' is used`
            });
        }
        else if (node.expression.type === 'ConditionalExpression') {
            for (const target of [
                node.expression.consequent,
                node.expression.alternate
            ]) {
                if (target.type !== 'Literal') {
                    continue;
                }
                if (isValidValue(target.value, config)) {
                    continue;
                }
                context.report({
                    node: target,
                    message: `raw text '${target.value}' is used`
                });
            }
        }
    }
}
function create(context) {
    var _a, _b, _c;
    const sourceCode = context.getSourceCode();
    const config = {
        ignorePattern: /^[^\S\s]$/,
        ignoreNodes: [],
        ignoreText: []
    };
    if ((_a = context.options[0]) === null || _a === void 0 ? void 0 : _a.ignorePattern) {
        config.ignorePattern = new RegExp(context.options[0].ignorePattern, 'u');
    }
    if ((_b = context.options[0]) === null || _b === void 0 ? void 0 : _b.ignoreNodes) {
        config.ignoreNodes = context.options[0].ignoreNodes;
    }
    if ((_c = context.options[0]) === null || _c === void 0 ? void 0 : _c.ignoreText) {
        config.ignoreText = context.options[0].ignoreText;
    }
    function isIgnore(node) {
        const element = getElement(node);
        return (!element ||
            config.ignoreNodes.includes(sourceCode.text.slice(...element.name.range)));
    }
    function getElement(node) {
        let target = node.parent;
        while (target.type === 'SvelteIfBlock' ||
            target.type === 'SvelteElseBlock' ||
            target.type === 'SvelteEachBlock' ||
            target.type === 'SvelteAwaitBlock' ||
            target.type === 'SvelteAwaitPendingBlock' ||
            target.type === 'SvelteAwaitThenBlock' ||
            target.type === 'SvelteAwaitCatchBlock' ||
            target.type === 'SvelteKeyBlock') {
            target = target.parent;
        }
        if (target.type === 'SvelteElement') {
            return target;
        }
        return null;
    }
    return {
        SvelteMustacheTag(node) {
            if (isIgnore(node)) {
                return;
            }
            checkSvelteMustacheTagText(context, node, config);
        },
        SvelteText(node) {
            if (isIgnore(node)) {
                return;
            }
            if (isValidValue(node.value, config)) {
                return;
            }
            context.report({
                node,
                message: `raw text '${node.value}' is used`
            });
        }
    };
}
module.exports = utils_1.defineRule('no-raw-text', {
    meta: {
        type: 'suggestion',
        docs: {
            description: 'disallow to string literal in template',
            category: 'Recommended',
            recommended: true
        },
        schema: [
            {
                type: 'object',
                properties: {
                    ignoreNodes: {
                        type: 'array'
                    },
                    ignorePattern: {
                        type: 'string'
                    },
                    ignoreText: {
                        type: 'array'
                    }
                }
            }
        ]
    },
    create
});
